@@ -4,7 +4,6 @@ Description :  Application-specific handler functions.
 Copyright   :  (c) 2011 Cedric Staub, 2012 Benedikt Schmidt
 License     :  GPL-3
 
-Maintainer  :  Cedric Staub <cstaub@ethz.ch>
 Stability   :  experimental
 Portability :  non-portable
 -}
@@ -33,6 +32,9 @@ module Web.Handler
   , getAutoProverR
   , getAutoDiffProverR
   , getAutoProverDiffR
+  , getAutoProverAllR
+  , getProverDiffAllR
+  , getAutoProverAllDiffR
   , getDeleteStepR
   , getDeleteStepDiffR
   , getKillThreadR
@@ -64,10 +66,12 @@ import           Theory                       (
     sorryDiffProver, runAutoDiffProver,
     prettyClosedTheory, prettyOpenTheory,
     openDiffTheory,
-    prettyClosedDiffTheory, prettyOpenDiffTheory
+    prettyClosedDiffTheory, prettyOpenDiffTheory, getLemmas, lName, lDiffName, getDiffLemmas, getEitherLemmas, thySignature, diffThySignature, toSignatureWithMaude
   )
-import           Theory.Proof (AutoProver(..), SolutionExtractor(..), Prover, DiffProver, apDefaultHeuristic)
-import           Text.PrettyPrint.Html
+
+import Debug.Trace
+import           Theory.Proof (AutoProver(..), SolutionExtractor(..), Prover, DiffProver)
+import Text.PrettyPrint.Html ( render, htmlDoc, renderHtmlDoc )
 import           Theory.Constraint.System.Dot
 import           Theory.Constraint.System.JSON  -- for export of constraint system to JSON
 import           Web.Hamlet
@@ -79,9 +83,8 @@ import           Web.Types
 import           Yesod.Core
 
 import           Control.Monad.Trans.Resource (runResourceT)
-import           Control.Monad.Trans.Unlift
 
-import           Data.Label
+import           Data.Label                   as L
 import           Data.Maybe
 import           Data.String                  (fromString)
 import           Data.List                    (intersperse)
@@ -103,14 +106,19 @@ import           Control.Applicative
 import           Control.Concurrent
 import qualified Control.Concurrent.Thread    as Thread ( forkIO )
 import           Control.DeepSeq
-import           Control.Exception.Base
-import qualified Control.Exception.Lifted     as E
+import           Control.Exception.Base       as E
 import           Control.Monad
 import qualified Data.Binary                  as Bin
 import           Data.Time.LocalTime
 import           System.Directory
 
-import           Debug.Trace                  (trace)
+import Control.Monad.Except (runExceptT)
+import Main.TheoryLoader
+import Main.Console (renderDoc)
+import Theory.Tools.Wellformedness (prettyWfErrorReport)
+import           Text.Read                    (readMaybe)
+import           Theory.Constraint.System.Graph.Graph
+import           Theory.Constraint.System.Dot (BoringNodeStyle, dotSystemCompact, doNodeStyle)
 
 -- Quasi-quotation syntax changed from GHC 6 to 7,
 -- so we need this switch in order to support both
@@ -147,8 +155,9 @@ getTheory idx = do
 putTheory :: Maybe TheoryInfo     -- ^ Index of parent theory
           -> Maybe TheoryOrigin         -- ^ Origin of this theory
           -> ClosedTheory         -- ^ The new closed theory
+          -> String
           -> Handler TheoryIdx
-putTheory parent origin thy = do
+putTheory parent origin thy rep = do
     yesod <- getYesod
     liftIO $ modifyMVar (theoryVar yesod) $ \theories -> do
       time <- getZonedTime
@@ -159,7 +168,7 @@ putTheory parent origin thy = do
           newOrigin    = parentOrigin <|> origin <|> (Just Interactive)
           newThy       = Trace (
               TheoryInfo idx thy time parentIdx False (fromJust newOrigin)
-                      (maybe (defaultAutoProver yesod) tiAutoProver parent))
+                      (maybe (defaultAutoProver yesod) tiAutoProver parent) rep)
       storeTheory yesod newThy idx
       return (M.insert idx newThy theories, idx)
 
@@ -167,8 +176,9 @@ putTheory parent origin thy = do
 putDiffTheory :: Maybe DiffTheoryInfo     -- ^ Index of parent theory
               -> Maybe TheoryOrigin         -- ^ Origin of this theory
               -> ClosedDiffTheory         -- ^ The new closed theory
+              -> String
               -> Handler TheoryIdx
-putDiffTheory parent origin thy = do
+putDiffTheory parent origin thy rep = do
     yesod <- getYesod
     liftIO $ modifyMVar (theoryVar yesod) $ \theories -> do
       time <- getZonedTime
@@ -179,7 +189,7 @@ putDiffTheory parent origin thy = do
           newOrigin    = parentOrigin <|> origin <|> (Just Interactive)
           newThy       = Diff (
               DiffTheoryInfo idx thy time parentIdx False (fromJust newOrigin)
-                      (maybe (defaultAutoProver yesod) dtiAutoProver parent))
+                      (maybe (defaultAutoProver yesod) dtiAutoProver parent) rep)
       storeTheory yesod newThy idx
       return (M.insert idx newThy theories, idx)
 
@@ -300,11 +310,11 @@ getThreads = do
 ------------------------------------------------------------------------------
 
 -- | Print exceptions, if they happen.
-traceExceptions :: MonadBaseControl IO m => String -> m a -> m a
+traceExceptions :: String -> IO a -> IO a
 traceExceptions info =
     E.handle handler
   where
-    handler :: MonadBaseControl IO m => E.SomeException -> m a
+    handler :: E.SomeException -> IO a
     handler e =
       trace (info ++ ": exception `" ++ show e ++ "'") $ E.throwIO e
 
@@ -329,7 +339,7 @@ responseToJson = go
 -- | Fully evaluate a value in a thread that can be canceled.
 evalInThread :: NFData a
              => IO a
-             -> Handler (Either SomeException a)
+             -> Handler (Either E.SomeException a)
 evalInThread io = do
     renderF <- getUrlRender
     maybeRoute <- getCurrentRoute
@@ -427,11 +437,12 @@ modifyTheory :: TheoryInfo                                -- ^ Theory to modify
              -> Handler Value
 modifyTheory ti f fpath errResponse = do
     res <- evalInThread (liftIO $ f (tiTheory ti))
+    let rep = tiErrorsHtml ti
     case res of
       Left e           -> return (excResponse e)
       Right Nothing    -> return (responseToJson errResponse)
       Right (Just thy) -> do
-        newThyIdx <- putTheory (Just ti) Nothing thy
+        newThyIdx <- putTheory (Just ti) Nothing thy rep
         newUrl <- getUrlRender <*> pure (OverviewR newThyIdx (fpath thy))
         return . responseToJson $ JsonRedirect newUrl
   where
@@ -446,11 +457,12 @@ modifyDiffTheory :: DiffTheoryInfo                                    -- ^ Theor
                  -> Handler Value
 modifyDiffTheory ti f fpath errResponse = do
     res <- evalInThread (liftIO $ f (dtiTheory ti))
+    rep <- pure $ dtiErrorsHtml ti
     case res of
       Left e           -> return (excResponse e)
       Right Nothing    -> return (responseToJson errResponse)
       Right (Just thy) -> do
-        newThyIdx <- putDiffTheory (Just ti) Nothing thy
+        newThyIdx <- putDiffTheory (Just ti) Nothing thy rep
         newUrl <- getUrlRender <*> pure (OverviewDiffR newThyIdx (fpath thy))
         return . responseToJson $ JsonRedirect newUrl
   where
@@ -485,27 +497,31 @@ postRootR = do
           content <- liftIO $ runResourceT $ C.runConduit (fileSource fileinfo C..| consume)
           if null content
             then setMessage "No theory file given."
-            else do
-              yesod <- getYesod
-              if isDiffTheory yesod
-                 then do
-                    closedThy <- liftIO $ diffParseThy yesod (T.unpack $ T.decodeUtf8 $ BS.concat content)
-                    case closedThy of
-                      Left err -> setMessage $ "Theory loading failed:\n" <> toHtml err
-                      Right thy -> do
-                          void $ putDiffTheory Nothing
-                                  (Just $ Upload $ T.unpack $ fileName fileinfo) thy
-                          wfReport <- liftIO $ thyWf yesod (T.unpack $ T.decodeUtf8 $ BS.concat content)
-                          setMessage $ toHtml $ "Loaded new theory!" ++  wfReport
-                 else do
-                    closedThy <- liftIO $ parseThy yesod (T.unpack $ T.decodeUtf8 $ BS.concat content)
-                    case closedThy of
-                      Left err -> setMessage $ "Theory loading failed:\n" <> toHtml err
-                      Right thy -> do
-                          void $ putTheory Nothing
-                                  (Just $ Upload $ T.unpack $ fileName fileinfo) thy
-                          wfReport <- liftIO $ thyWf yesod (T.unpack $ T.decodeUtf8 $ BS.concat content)
-                          setMessage $ toHtml $ "Loaded new theory!" ++ wfReport
+          else do
+            yesod <- getYesod
+            thyWithRep <- liftIO $ runExceptT $ do
+              openThy <- loadThy yesod (T.unpack $ T.decodeUtf8 $ BS.concat content) (T.unpack $ fileName fileinfo)
+
+              let sig = either (get thySignature) (get diffThySignature) openThy
+              sig'   <- liftIO $ toSignatureWithMaude yesod.thyOpts.maudePath sig
+
+              -- let tactic = get thyTactic openThy
+              --tactic'   <- liftIO $ toSignatureWithMaude (get oMaudePath (thyOpts yesod)) tactic
+
+              closeThy yesod sig' openThy
+
+            case thyWithRep of
+              Left err -> setMessage $ "Theory loading failed:\n" <> toHtml (show err)
+              Right (report, thy) -> do
+                wfErrors <- case report of
+                  [] -> pure $ ""
+                  _ -> pure $ "<div class=\"wf-warning\">\nWARNING: the following wellformedness checks failed!<br /><br />\n" ++ (renderHtmlDoc . htmlDoc $ prettyWfErrorReport report) ++ "\n</div>"
+                void $ either (putTheory Nothing (Just $ Upload $ T.unpack $ fileName fileinfo))
+                              (putDiffTheory Nothing (Just $ Upload $ T.unpack $ fileName fileinfo)) thy wfErrors
+                setMessage $ toHtml $ "Loaded new theory!" ++
+                                      " WARNING: ignoring the following wellformedness errors: " ++
+                                      renderDoc (prettyWfErrorReport report)
+
     theories <- getTheories
     defaultLayout $ do
       setTitle "Welcome to the Tamarin prover"
@@ -516,8 +532,11 @@ postRootR = do
 getOverviewR :: TheoryIdx -> TheoryPath -> Handler Html
 getOverviewR idx path = withTheory idx ( \ti -> do
   renderF <- getUrlRender
+  renderParamsF <- getUrlRenderParams
   defaultLayout $ do
-    overview <- liftIO $ overviewTpl renderF ti path
+    getParams <- reqGetParams <$> getRequest
+    let renderParamsF' route = renderParamsF route getParams
+    overview <- liftIO $ overviewTpl renderF renderParamsF' ti path
     setTitle (toHtml $ "Theory: " ++ get thyName (tiTheory ti))
     overview )
 
@@ -594,18 +613,16 @@ getTheoryPathMR idx path = do
     -- Handle method paths by trying to solve the given goal/method
     --
     go _ (TheoryMethod lemma proofPath i) ti = modifyTheory ti
-        (\thy -> return $ applyMethodAtPath thy lemma proofPath heuristic i)
+        (\thy -> return $ applyMethodAtPath thy lemma proofPath (tiAutoProver ti) i)
         (\thy -> nextSmartThyPath thy (TheoryProof lemma proofPath))
         (JsonAlert "Sorry, but the prover failed on the selected method!")
-      where
-        heuristic = apDefaultHeuristic (tiAutoProver ti)
 
     --
     -- Handle generic paths by trying to render them
     --
     go renderUrl _ ti = do
       let title = T.pack $ titleThyPath (tiTheory ti) path
-      let html = htmlThyPath renderUrl ti path
+      let html = htmlThyPath renderUrl renderUrl ti path
       return $ responseToJson (JsonHtml title $ toContent html)
 
 -- | Show a given path within a diff theory (main view).
@@ -622,17 +639,13 @@ getTheoryPathDiffMR idx path = do
     -- Handle method paths by trying to solve the given goal/method
     --
     goDiff _ (DiffTheoryMethod s lemma proofPath i) ti = modifyDiffTheory ti
-        (\thy -> return $ applyMethodAtPathDiff thy s lemma proofPath heuristic i)
+        (\thy -> return $ applyMethodAtPathDiff thy s lemma proofPath (dtiAutoProver ti) i)
         (\thy -> nextSmartDiffThyPath thy (DiffTheoryProof s lemma proofPath))
         (JsonAlert "Sorry, but the prover failed on the selected method!")
-      where
-        heuristic = apDefaultHeuristic (dtiAutoProver ti)
     goDiff _ (DiffTheoryDiffMethod lemma proofPath i) ti = modifyDiffTheory ti
-        (\thy -> return $ applyDiffMethodAtPath thy lemma proofPath heuristic i)
+        (\thy -> return $ applyDiffMethodAtPath thy lemma proofPath (dtiAutoProver ti) i)
         (\thy -> nextSmartDiffThyPath thy (DiffTheoryDiffProof lemma proofPath))
         (JsonAlert "Sorry, but the prover failed on the selected method!")
-      where
-        heuristic = apDefaultHeuristic (dtiAutoProver ti)
 
     --
     -- Handle generic paths by trying to render them
@@ -661,6 +674,22 @@ getProverR (name, mkProver) idx path = do
     go _ _ = return $ responseToJson $ JsonAlert $
       "Can't run " <> name <> " on the given theory path!"
 
+-- | Run the some prover on a given proof path.
+getProverAllR :: (T.Text, AutoProver -> Prover)
+           -> TheoryIdx -> Handler RepJson
+getProverAllR (name, mkProver) idx = do
+    jsonValue <- withTheory idx go
+    return $ RepJson $ toContent jsonValue
+  where
+    go ti = modifyTheory ti
+        proveAll
+        (\thy -> nextSmartThyPath thy (TheoryProof (last $ names thy) []))
+        (JsonAlert $ "Sorry, but " <> name <> " failed!")
+      where
+        names thy = map (get lName) $ getLemmas thy
+        autoProver = mkProver (tiAutoProver ti)
+        proveAll thy = return $ foldM (\tha lemma -> applyProverAtPath tha lemma [] autoProver) thy $ names thy
+
 -- | Run the some prover on a given proof path.
 getProverDiffR :: (T.Text, AutoProver -> Prover)
                -> TheoryIdx -> Side -> DiffTheoryPath -> Handler RepJson
@@ -703,14 +732,60 @@ getDiffProverR (name, mkProver) idx path = do
     goDiff _ _ = return $ responseToJson $ JsonAlert $
       "Can't run " <> name <> " on the given theory path!"
 
+-- | Run the some prover on a given proof path.
+getProverDiffAllR :: (T.Text, AutoProver -> Prover ,AutoProver -> DiffProver)
+           -> TheoryIdx -> Handler RepJson
+getProverDiffAllR (name, mkProver, mkDiffProver) idx  = do
+    jsonValue <- withDiffTheory idx goDiff
+    return $ RepJson $ toContent jsonValue
+  where
+    goDiff ti = modifyDiffTheory ti
+        proveAllDiff
+        (\thy -> nextSmartDiffThyPath thy (DiffTheoryDiffProof (last $ namesDiff thy) []))
+        (JsonAlert $ "Sorry, but " <> name <> " failed!")
+      where
+        namesDiff thy = map (get lDiffName) $ getDiffLemmas thy
+        autoDiffProver = mkDiffProver (dtiAutoProver ti)
+        names thy = map (\(x, y) -> (x, get lName y)) $ getEitherLemmas thy
+        autoProver = mkProver (dtiAutoProver ti)
+        proveDiff thy = foldM (\tha lemma -> applyDiffProverAtPath tha lemma [] autoDiffProver) thy $ namesDiff thy
+        proveAllDiff thy = return $ (proveDiff thy) >>= (\thb -> foldM (\tha (s, lemma) -> applyProverAtPathDiff tha s lemma [] autoProver) thb $ names thb)
 
 -- | Run an autoprover on a given proof path.
 getAutoProverR :: TheoryIdx
                -> SolutionExtractor
                -> Int                             -- autoprover bound to use
+               -> Bool                            -- Quit on empty oracle
                -> TheoryPath -> Handler RepJson
-getAutoProverR idx extractor bound =
+getAutoProverR idx extractor bound quitOnEmpty =
     getProverR (fullName, runAutoProver . adapt) idx
+  where
+    adapt autoProver = autoProver
+      { apBound = actualBound
+      , apCut = if quitOnEmpty then CutAfterSorry else extractor
+      , quitOnEmptyOracle = quitOnEmpty }
+
+    withCommas = intersperse ", "
+    fullName   = mconcat $ proverName : " (" : withCommas qualifiers ++ [")"]
+    qualifiers = extractorQualfier ++ boundQualifier
+
+    (actualBound, boundQualifier)
+        | bound > 0 = (Just bound, ["bound " <> T.pack (show bound)])
+        | otherwise = (Nothing,    []                               )
+
+    (proverName, extractorQualfier) = case extractor of
+        CutNothing         -> ("characterization", ["dfs"]   )
+        CutDFS             -> ("the autoprover",   []        )
+        CutBFS             -> ("the autoprover",   ["bfs"]   )
+        CutSingleThreadDFS -> ("the autoprover",   ["seqdfs"])
+
+-- | Run an autoprover on a given proof path.
+getAutoProverAllR :: TheoryIdx
+               -> SolutionExtractor
+               -> Int                             -- autoprover bound to use
+               -> TheoryPath -> Handler RepJson
+getAutoProverAllR idx extractor bound _ =
+    getProverAllR (fullName, runAutoProver . adapt) idx
   where
     adapt autoProver = autoProver { apBound = actualBound, apCut = extractor }
 
@@ -728,6 +803,7 @@ getAutoProverR idx extractor bound =
         CutBFS             -> ("the autoprover",   ["bfs"]   )
         CutSingleThreadDFS -> ("the autoprover",   ["seqdfs"])
 
+
 -- | Run an autoprover on a given proof path.
 getAutoProverDiffR :: TheoryIdx
                    -> SolutionExtractor
@@ -752,6 +828,32 @@ getAutoProverDiffR idx extractor bound s =
         CutBFS             -> ("the autoprover",   ["bfs"]   )
         CutSingleThreadDFS -> ("the autoprover",   ["seqdfs"])
 
+
+-- | Run an autoprover on a given proof path.
+getAutoProverAllDiffR :: TheoryIdx
+               -> SolutionExtractor
+               -> Int                             -- autoprover bound to use
+               -> Handler RepJson
+getAutoProverAllDiffR idx extractor bound =
+    getProverDiffAllR (fullName, runAutoProver . adapt, runAutoDiffProver . adapt) idx
+  where
+    adapt autoProver = autoProver { apBound = actualBound, apCut = extractor }
+
+    withCommas = intersperse ", "
+    fullName   = mconcat $ proverName : " (" : withCommas qualifiers ++ [")"]
+    qualifiers = extractorQualfier ++ boundQualifier
+
+    (actualBound, boundQualifier)
+        | bound > 0 = (Just bound, ["bound " <> T.pack (show bound)])
+        | otherwise = (Nothing,    []                               )
+
+    (proverName, extractorQualfier) = case extractor of
+        CutNothing         -> ("characterization", ["dfs"]   )
+        CutDFS             -> ("the autoprover",   []        )
+        CutBFS             -> ("the autoprover",   ["bfs"]   )
+        CutSingleThreadDFS -> ("the autoprover",   ["seqdfs"])
+
+
 -- | Run an autoprover on a given proof path.
 getAutoDiffProverR :: TheoryIdx
                    -> SolutionExtractor
@@ -803,31 +905,43 @@ getTheoryPathDR idx path = withTheory idx $ \ti -> ajaxLayout $ do
   -}
 -}
 
+-- | Read the render options from a request to render a sequent.
+getOptions :: Handler (GraphOptions, DotOptions)
+getOptions = do
+  compact <- isNothing <$> lookupGetParam "uncompact"
+  let nodeStyle = if compact then CompactBoringNodes else FullBoringNodes
+  compress <- isNothing <$> lookupGetParam "uncompress"
+  abbreviate <- isNothing <$> lookupGetParam "unabbreviate"
+  simpl <- lookupGetParam "simplification"
+  showAutosource <- isNothing <$> lookupGetParam "no-auto-sources"
+  clustering <- lookupGetParam "clustering"
+  let simplificationLevel = fromMaybe SL2 (simpl >>= readMaybe . T.unpack) 
+      graphOptions = L.set goSimplificationLevel simplificationLevel $
+                     L.set goCompress compress $
+                     L.set goShowAutoSource showAutosource $
+                     L.set goAbbreviate abbreviate $
+                     L.set goClustering (isJust clustering) $ 
+                     defaultGraphOptions
+  let dotOptions = L.set doNodeStyle nodeStyle defaultDotOptions
+  return (graphOptions, dotOptions)
+
+
 -- | Get rendered graph for theory and given path.
 getTheoryGraphR :: TheoryIdx -> TheoryPath -> Handler ()
 getTheoryGraphR idx path = withTheory idx ( \ti -> do
       yesod <- getYesod
-      compact <- isNothing <$> lookupGetParam "uncompact"
-      compress <- isNothing <$> lookupGetParam "uncompress"
-      abbreviate <- isNothing <$> lookupGetParam "unabbreviate"
-      simplificationLevel <- fromMaybe "1" <$> lookupGetParam "simplification"
-      img <- liftIO $ traceExceptions "getTheoryGraphR" $
+      (graphOptions, dotOptions) <- getOptions
+      img' <- liftIO $ traceExceptions "getTheoryGraphR" $
         imgThyPath
           (imageFormat yesod)
-          (graphCmd yesod)
+          (outputCmd yesod)
           (cacheDir yesod)
-          (graphStyle compact compress)
-          (sequentToJSONPretty)
-          (show simplificationLevel)
-          (abbreviate)
+          (dotSystemCompact graphOptions dotOptions)
+          (\label system -> sequentsToJSONPretty graphOptions [(label, system)])
           (tiTheory ti) path
-      sendFile (fromString . imageFormatMIME $ imageFormat yesod) img)
-  where
-    graphStyle d c = dotStyle d . compression c
-    dotStyle True = dotSystemCompact CompactBoringNodes
-    dotStyle False = dotSystemCompact FullBoringNodes
-    compression True = compressSystem
-    compression False = id
+      case img' of
+        Nothing -> notFound
+        Just img -> sendFile (fromString . imageFormatMIME $ imageFormat yesod) img)
 
 -- | Get rendered graph for theory and given path.
 getTheoryGraphDiffR :: TheoryIdx -> DiffTheoryPath -> Handler ()
@@ -837,27 +951,19 @@ getTheoryGraphDiffR idx path = getTheoryGraphDiffR' idx path False
 getTheoryGraphDiffR' :: TheoryIdx -> DiffTheoryPath -> Bool -> Handler ()
 getTheoryGraphDiffR' idx path mirror = withDiffTheory idx ( \ti -> do
       yesod <- getYesod
-      compact <- isNothing <$> lookupGetParam "uncompact"
-      compress <- isNothing <$> lookupGetParam "uncompress"
-      abbreviate <- isNothing <$> lookupGetParam "unabbreviate"
-      simplificationLevel <- fromMaybe "1" <$> lookupGetParam "simplification"
-      img <- liftIO $ traceExceptions "getTheoryGraphDiffR" $
+      (graphOptions, dotOptions) <- getOptions
+      img' <- liftIO $ traceExceptions "getTheoryGraphDiffR" $
         imgDiffThyPath
           (imageFormat yesod)
-          (snd $ graphCmd yesod)
+          -- a.d. TODO should diff theories support JSON output?
+          (ocGraphCommand $ outputCmd yesod)
           (cacheDir yesod)
-          (graphStyle compact compress)
-          (show simplificationLevel)
-          (abbreviate)
+          (dotSystemCompact graphOptions dotOptions)
           (dtiTheory ti) path
           (mirror)
-      sendFile (fromString . imageFormatMIME $ imageFormat yesod) img)
-  where
-    graphStyle d c = dotStyle d . compression c
-    dotStyle True = dotSystemCompact CompactBoringNodes
-    dotStyle False = dotSystemCompact FullBoringNodes
-    compression True = compressSystem
-    compression False = id
+      case img' of
+        Nothing -> notFound
+        Just img -> sendFile (fromString . imageFormatMIME $ imageFormat yesod) img)
 
 -- | Get rendered mirror graph for theory and given path.
 getTheoryMirrorDiffR :: TheoryIdx -> DiffTheoryPath -> Handler ()
@@ -1119,7 +1225,7 @@ getSaveTheoryR idx = withEitherTheory idx $ \eti -> do
               -- Return message
               jsonResp (JsonAlert $ T.pack $ "Saved theory to file: " ++ file)
   where
-    prettyRender ti  = render $ prettyOpenTheory $ openTheory $ tiTheory ti 
+    prettyRender ti  = render $ prettyOpenTheory $ openTheory $ tiTheory ti
     prettyRenderD ti = render $ prettyOpenDiffTheory $ openDiffTheory $ dtiTheory ti
     same origin (Trace ti) = tiPrimary ti  && (tiOrigin ti  == origin)
     same origin (Diff ti)    = dtiPrimary ti && (dtiOrigin ti == origin)
